# -*- Mode: Python; -*-
# vi:si:et:sw=4:sts=4:ts=4
#

# U8MG - an utf8 migration tool
#
# Copyright (C) 2005 Tollef Fog Heen
#
# Wizards taken from flumotion,
# Copyright (C) 2004,2005 Fluendo, S.L. (www.fluendo.com). All rights reserved.

# This file may be distributed and/or modified under the terms of
# the GNU General Public License version 2 as published by
# the Free Software Foundation.
# This file is distributed without any warranty; without even the implied
# warranty of merchantability or fitness for a particular purpose.
# See "LICENSE.GPL" in the source distribution for more information.

